@@ -5,8 +5,8 @@
  *
  *      Description:  Implementation of access daemon.
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Authors:  Michael Meier, michael.meier@rrze.fau.de
  *                Jan Treibig (jt), jan.treibig@gmail.com,
@@ -110,14 +110,16 @@ static int num_pmc_counters = 0;
 
 /* Socket to bus mapping -- will be determined at runtime;
  * typical mappings are:
- * Socket  Bus (2S)  Bus (4s)
- *   0        0xff      0x3f
- *   1        0x7f      0x7f
- *   2                  0xbf
- *   3                  0xff
+ * Socket  Bus (2S)  Bus (4s)  Bus (8s)
+ *   0        0xff      0x3f     0x1f
+ *   1        0x7f      0x7f     0x3f
+ *   2                  0xbf     0x5f
+ *   3                  0xff     0x7f
+ *                               0x9f
+ *                               0xbf
+ *                               0xdf
+ *                               0xff
  */
-//static char* socket_bus[MAX_NUM_NODES] = { [0 ... (MAX_NUM_NODES-1)] = NULL};
-static char** socket_bus = NULL;
 static int avail_sockets = 0;
 static int avail_cpus = 0;
 
@@ -152,9 +154,6 @@ void __attribute__((constructor (101))) init_accessdaemon(void)
     FD_MSR = malloc(avail_cpus * sizeof(int));
     //memset(FD_MSR, 0, avail_cpus * sizeof(int));
 
-    socket_bus = malloc(avail_sockets * sizeof(char*));
-    //memset(socket_bus, 0, avail_sockets * sizeof(char*));
-
     FD_PCI = malloc(avail_sockets * sizeof(int*));
     //memset(FD_PCI, 0, avail_sockets * sizeof(int*));
     for (int i = 0; i < avail_sockets; i++)
@@ -173,8 +172,7 @@ void __attribute__((destructor (101))) close_accessdaemon(void)
     }
     free(FD_PCI);
     FD_PCI = NULL;
-    free(socket_bus);
-    socket_bus = NULL;
+
     free(FD_MSR);
     FD_MSR = NULL;
 }
@@ -843,8 +841,8 @@ msr_read(AccessDataRecord * dRecord)
 
     if (pread(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to read data from register 0x%x on core %u", reg, cpu);
 #ifdef DEBUG_LIKWID
+        syslog(LOG_ERR, "Failed to read data from register 0x%x on core %u", reg, cpu);
         syslog(LOG_ERR, "%s", strerror(errno));
 #endif
         dRecord->errorcode = ERR_RWFAIL;
@@ -884,8 +882,8 @@ msr_write(AccessDataRecord * dRecord)
 
     if (pwrite(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to write data to register 0x%x on core %u", reg, cpu);
 #ifdef DEBUG_LIKWID
+        syslog(LOG_ERR, "Failed to write data to register 0x%x on core %u", reg, cpu);
         syslog(LOG_ERR, "%s", strerror(errno));
 #endif
         dRecord->errorcode = ERR_RWFAIL;
@@ -907,59 +905,217 @@ msr_check(AccessDataRecord * dRecord)
     return;
 }
 
-static int
-getBusFromSocket(PciDevice* dev, int socket, char** filepath)
+
+#define PCI_SLOT(devfn)         (((devfn) >> 3) & 0x1f)
+#define PCI_FUNC(devfn)         ((devfn) & 0x07)
+/* This code uses the filename and the devid specified in the PciDevice list.
+ * It traverses sysfs because the sysfs contains (commonly) the node ID the PCI
+ * device is attached to. If the numa_node == -1 it uses a counter to get the
+ * node ID
+ */
+static int getBusFromSocketByNameDevid(const uint32_t socket, PciDevice* pcidev, int pathlen, char** filepath)
 {
-    int cur_bus = 0;
-    uint32_t cur_socket = socket;
-    char pci_filepath[1024];
-    int fp;
-    int ret = 0;
-    while(socket >= 0)
+    struct dirent *pDirent, *pDirentInner;
+    DIR *pDir, *pDirInner;
+    FILE* fp = NULL;
+    int tmplen = 200;
+    char tmpPath[200], buff[200];
+    size_t ret = 0;
+    int bus_id = -1;
+    int numa_ctr = 0;
+
+    pDir = opendir ("/sys/devices");
+    if (pDir == NULL)
     {
-        while (cur_bus <= 255)
+        syslog(LOG_ERR, "Failed open directory /sys/devices\n");
+        return -1;
+    }
+
+    while ((pDirent = readdir(pDir)) != NULL)
+    {
+        if (strncmp(pDirent->d_name, "pci0", 4) == 0)
         {
-            snprintf(pci_filepath, 1023, "/proc/bus/pci/%02x/%s", cur_bus, dev->path);
-            fp = open(pci_filepath, O_RDONLY);
-            if (fp < 0)
+            memset(tmpPath, '\0', tmplen*sizeof(char));
+            sprintf(tmpPath, "/sys/devices/%s", pDirent->d_name);
+
+            char bus[4];
+            strncpy(bus, &(pDirent->d_name[strlen(pDirent->d_name)-2]), 2);
+            bus[2] = '\0';
+
+            pDirInner = opendir (tmpPath);
+            if (pDir == NULL)
             {
-                cur_bus++;
-                continue;
+                syslog(LOG_ERR, "Failed read file %s\n", tmpPath);
+                return -1;
             }
-            if (fp > 0)
+            while ((pDirentInner = readdir(pDirInner)) != NULL)
             {
-                uint32_t indev = 0;
-                ret = pread(fp, &indev, sizeof(uint32_t), 0x0);
-                indev = (indev >> 16);
-                if (indev != dev->devid)
+                if (strncmp(pDirentInner->d_name, "0000", 4) == 0)
                 {
-                    cur_bus++;
-                    continue;
+                    uint32_t dev_id = 0x0;
+                    int numa_node = 0;
+                    memset(tmpPath, '\0', tmplen*sizeof(char));
+                    sprintf(tmpPath, "/sys/devices/%s/%s/device", pDirent->d_name, pDirentInner->d_name);
+                    if (pcidev->path && strcmp(&(pDirentInner->d_name[strlen(pDirentInner->d_name)-4]), pcidev->path) != 0)
+                    {
+                        continue;
+                    }
+                    fp = fopen(tmpPath,"r");
+                    if( fp != NULL )
+                    {
+                        memset(buff, '\0', tmplen*sizeof(char));
+                        ret = fread(buff, sizeof(char), tmplen-1, fp);
+                        fclose(fp);
+                        if (ret > 0)
+                        {
+                            dev_id = strtoul(buff, NULL, 16);
+
+                            if (dev_id == pcidev->devid)
+                            {
+                                memset(tmpPath, '\0', tmplen*sizeof(char));
+                                sprintf(tmpPath, "/sys/devices/%s/%s/numa_node", pDirent->d_name, pDirentInner->d_name);
+                                syslog(LOG_ERR, "Read file %s\n", tmpPath);
+                                fp = fopen(tmpPath,"r");
+                                if( fp != NULL )
+                                {
+                                    memset(buff, '\0', tmplen*sizeof(char));
+                                    ret = fread(buff, sizeof(char), tmplen-1, fp);
+                                    numa_node = atoi(buff);
+                                    if (numa_node < 0)
+                                    {
+                                        numa_node = numa_ctr;
+                                        numa_ctr++;
+                                    }
+                                    if (numa_node == socket)
+                                    {
+                                        bus_id = strtoul(bus, NULL, 16);
+                                        if (filepath && *filepath && pathlen > 0)
+                                            snprintf(*filepath, pathlen-1, "/proc/bus/pci/%02x/%s", bus_id, pcidev->path);
+                                        break;
+                                    }
+                                }
+                            }
+                        }
+                    }
                 }
             }
-            if (fp > 0 && socket > 0)
+            closedir (pDirInner);
+            if (bus_id != -1)
+                break;
+        }
+    }
+    closedir (pDir);
+#ifdef DEBUG_LIKWID
+    syslog(LOG_INFO, "PCI device %s (%s) for socket %d is on PCI bus 0x%02x", pcidev->likwid_name, pcidev->path, socket, bus_id);
+#endif
+    return bus_id;
+}
+
+
+
+/* This code gets the PCI device using the given devid in pcidev. It assumes that the
+ * PCI busses are sorted like: if sock_id1 < sock_id2 then bus1 < bus2 end
+ * This code is only the fallback if a device is not found using the combination of
+ * filename and devid
+ */
+typedef struct {
+    uint32_t bus;
+    uint32_t devfn;
+} PciCandidate;
+
+static int getBusFromSocketByDevid(const uint32_t socket, PciDevice* pcidev, int pathlen, char** filepath)
+{
+    int ret = 0;
+    int cur_socket = (int)socket;
+    int out_bus_id = -1;
+    uint32_t out_devfn = 0x0;
+    int bufflen = 1024;
+    char buff[1024];
+    FILE* fp = NULL;
+    uint32_t bus, devfn, vendor, devid;
+    PciCandidate candidates[10];
+    int candidate = -1;
+    int cand_idx = 0;
+
+    fp = fopen("/proc/bus/pci/devices", "r");
+    if (fp)
+    {
+        while (fgets(buff, bufflen, fp) != NULL)
+        {
+            ret = sscanf((char*)buff, "%02x%02x\t%04x%04x", &bus, &devfn, &vendor, &devid);
+            if (ret == 4 && devid == pcidev->devid)
             {
-                cur_bus++;
-                socket--;
-                close(fp);
-                continue;
+                candidates[cand_idx].bus = bus;
+                candidates[cand_idx].devfn = devfn;
+                cand_idx++;
             }
-            
+        }
+        fclose(fp);
+    }
+    else
+    {
+        syslog(LOG_ERR, "Failed read file /proc/bus/pci/devices\n");
+    }
+
+    while (cur_socket >= 0)
+    {
+        int min_idx = 0;
+        uint32_t min = 0xFFF;
+        for (ret = 0; ret < cand_idx; ret++)
+        {
+            if (candidates[ret].bus < min)
+            {
+                min = candidates[ret].bus;
+                min_idx = ret;
+            }
+        }
+        if (cur_socket > 0)
+        {
+            candidates[min_idx].bus = 0xFFF;
+            cur_socket--;
+        }
+        else
+        {
+            if (candidates[min_idx].bus <= 0xff)
+            {
+                candidate = min_idx;
+            }
+            cur_socket = -1;
             break;
         }
-        socket--;
     }
-    if (fp > 0)
+
+    if (filepath && *filepath && pathlen > 0 && candidate >= 0 && candidates[candidate].bus >= 0 && candidates[candidate].bus <= 0xff)
     {
-        if (filepath)
-            strncpy(*filepath, pci_filepath, strlen(pci_filepath));
-        close(fp);
-        return cur_socket;
+        snprintf(*filepath, pathlen-1, "/proc/bus/pci/%02x/%02x.%01x", candidates[candidate].bus, PCI_SLOT(candidates[candidate].devfn), PCI_FUNC(candidates[candidate].devfn));
     }
+
+    if (candidate >= 0 && candidates[candidate].bus > 0 && candidates[candidate].devfn > 0)
+        return candidates[candidate].bus;
     return -1;
 }
 
 
+static int getBusFromSocket(const uint32_t socket, PciDevice* pcidev, int pathlen, char** filepath)
+{
+    int ret_sock = -1;
+    int ret = getBusFromSocketByNameDevid(socket, pcidev, pathlen, filepath);
+    if (ret < 0)
+    {
+        ret = getBusFromSocketByDevid(socket, pcidev, pathlen, filepath);
+        if (ret > 0)
+        {
+            ret_sock = socket;
+        }
+    }
+    else
+    {
+        ret_sock = socket;
+    }
+    return ret_sock;
+}
+
+
 static void
 pci_read(AccessDataRecord* dRecord)
 {
@@ -996,10 +1152,9 @@ pci_read(AccessDataRecord* dRecord)
     }
     if ( !FD_PCI[socketId][device] )
     {
-        //snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
         pcipath = malloc(1024 * sizeof(char));
         memset(pcipath, '\0', 1024 * sizeof(char));
-        int sid = getBusFromSocket(&(pci_devices_daemon[device]), socketId, &pcipath);
+        int sid = getBusFromSocket(socketId, &(pci_devices_daemon[device]), 1024, &pcipath);
         if (sid == socketId)
         {
 #ifdef DEBUG_LIKWID
@@ -1087,10 +1242,9 @@ pci_write(AccessDataRecord* dRecord)
 
     if ( !FD_PCI[socketId][device] )
     {
-        //snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
         pcipath = malloc(1024 * sizeof(char));
         memset(pcipath, '\0', 1024 * sizeof(char));
-        int sid = getBusFromSocket(&(pci_devices_daemon[device]), socketId, &pcipath);
+        int sid = getBusFromSocket(socketId, &(pci_devices_daemon[device]), 1024, &pcipath);
         if (sid == socketId)
         {
 #ifdef DEBUG_LIKWID
@@ -1169,13 +1323,6 @@ static void
 stop_daemon(void)
 {
     kill_client();
-    for (int i=0;i<MAX_NUM_NODES;i++)
-    {
-        if (socket_bus[i] != NULL)
-        {
-            free(socket_bus[i]);
-        }
-    }
 
     if (sockfd != -1)
     {
@@ -1187,127 +1334,6 @@ stop_daemon(void)
     exit(EXIT_SUCCESS);
 }
 
-static int
-getBusFromSocketOld(const uint32_t socket, const char* name, const uint32_t in_dev_id)
-{
-    int cur_bus = 0;
-    uint32_t cur_socket = 0;
-    char pci_filepath[1024];
-    int fp;
-    int ret = 0;
-    while(cur_socket <= socket)
-    {
-        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%02x/%s", PCI_ROOT_PATH, cur_bus, name);
-        fp = open(pci_filepath, O_RDONLY);
-        if (fp < 0)
-        {
-            return -1;
-        }
-        uint32_t cpubusno = 0;
-        ret = pread(fp, &cpubusno, sizeof(uint32_t), 0x108);
-        if (ret != sizeof(uint32_t))
-        {
-            close(fp);
-            return -1;
-        }
-        cur_bus = (cpubusno >> 8) & 0x0ff;
-        close(fp);
-        if(socket == cur_socket)
-            return cur_bus;
-        ++cur_socket;
-        ++cur_bus;
-        if(cur_bus > 0x0ff)
-           return -1;
-    }
-
-    return -1;
-}
-
-#define SKYLAKE_SERVER_SOCKETID_MBOX_DID 0x2042
-static int getBusFromSocketSysFS(const uint32_t socket, const char* name, const uint32_t in_dev_id)
-{
-    struct dirent *pDirent, *pDirentInner;
-    DIR *pDir, *pDirInner;
-    pDir = opendir ("/sys/devices");
-    FILE* fp = NULL;
-    char iPath[200], iiPath[200], buff[100];
-    char testDev[50];
-    size_t ret = 0;
-    int bus_id = -1;
-    int numa_ctr = 0;
-    if (pDir == NULL)
-    {
-        syslog(LOG_ERR, "Failed open directory /sys/devices");
-        return -1;
-    }
-    
-    while ((pDirent = readdir(pDir)) != NULL)
-    {
-        if (strncmp(pDirent->d_name, "pci0", 4) == 0)
-        {
-            sprintf(iPath, "/sys/devices/%s", pDirent->d_name);
-            char bus[4];
-            strncpy(bus, &(pDirent->d_name[strlen(pDirent->d_name)-2]), 2);
-            bus[2] = '\0';
-            
-            pDirInner = opendir (iPath);
-            if (pDir == NULL)
-            {
-                syslog(LOG_ERR, "Failed read file %s", iPath);
-                return -1;
-            }
-            while ((pDirentInner = readdir(pDirInner)) != NULL)
-            {
-                if (strncmp(pDirentInner->d_name, "0000", 4) == 0)
-                {
-                    uint32_t dev_id = 0x0;
-                    int numa_node = 0;
-                    sprintf(iiPath, "/sys/devices/%s/%s/device", pDirent->d_name, pDirentInner->d_name);
-                    fp = fopen(iiPath,"r");
-                    if( fp == NULL )
-                    {
-                        continue;
-                    }
-                    ret = fread(buff, sizeof(char), 99, fp);
-                    dev_id = strtoul(buff, NULL, 16);
-                    if (dev_id == in_dev_id)
-                    {
-                        fclose(fp);
-                        iiPath[0] = '\0';
-                        sprintf(iiPath, "/sys/devices/%s/%s/numa_node", pDirent->d_name, pDirentInner->d_name);
-                        fp = fopen(iiPath,"r");
-                        if( fp == NULL )
-                        {
-                            continue;
-                        }
-                        ret = fread(buff, sizeof(char), 99, fp);
-                        numa_node = atoi(buff);
-                        if (numa_node < 0)
-                        {
-                            numa_node = numa_ctr;
-                            numa_ctr++;
-                        }
-                        if (numa_node == socket)
-                        {
-                            bus_id = strtoul(bus, NULL, 16);
-                        }
-                    }
-                    fclose(fp);
-                    iiPath[0] = '\0';
-                    buff[0] = '\0';
-                    if (bus_id != -1)
-                        break;
-                }
-            }
-            closedir (pDirInner);
-            iPath[0] = '\0';
-            if (bus_id != -1)
-                break;
-        }
-    }
-    closedir (pDir);
-    return bus_id;
-}
 
 
 static void
@@ -1402,7 +1428,7 @@ int main(void)
         syslog(LOG_ERR,"Access to performance counters is locked.\n");
         stop_daemon();
     }
-    if (!FD_MSR || !FD_PCI || !socket_bus)
+    if (!FD_MSR || !FD_PCI)
     {
         syslog(LOG_ERR,"Failed to allocate FD space.\n");
         stop_daemon();
@@ -1492,7 +1518,8 @@ int main(void)
                 {
                     allowed = allowed_silvermont;
                 }
-                else if (model == XEON_PHI_KNL)
+                else if ((model == XEON_PHI_KNL) ||
+                         (model == XEON_PHI_KML))
                 {
                     allowed = allowed_knl;
                     isPCIUncore = 1;
@@ -1634,7 +1661,8 @@ int main(void)
                 //testDevice = 0x80862f30;
                 pci_devices_daemon = skylakeX_pci_devices;
             }
-            else if (model == XEON_PHI_KNL)
+            else if ((model == XEON_PHI_KNL) ||
+                     (model == XEON_PHI_KML))
             {
                 pci_devices_daemon = knl_pci_devices;
             }
@@ -1647,8 +1675,6 @@ int main(void)
 
             for (int j=0; j<avail_sockets; j++)
             {
-                //socket_bus[j] = (char*)malloc(4);
-                //sprintf(socket_bus[j], "N-A");
                 for (int i=0; i<MAX_NUM_PCI_DEVICES; i++)
                 {
                     FD_PCI[j][i] = -2;
@@ -1659,7 +1685,7 @@ int main(void)
             {
                 if (pci_devices_daemon[i].path && strlen(pci_devices_daemon[i].path) > 0)
                 {
-                    int socket_id = getBusFromSocket(&(pci_devices_daemon[i]), 0, NULL);
+                    int socket_id = getBusFromSocket(0, &(pci_devices_daemon[i]), 0, NULL);
                     if (socket_id == 0)
                     {
                         for (int j=0; j<avail_sockets; j++)
@@ -1674,53 +1700,6 @@ int main(void)
                     }
                 }
             }
-
-            /* determine PCI-BUSID mapping ... */
-            /*syslog(LOG_ERR, "Determine PCI-BUSID mapping");
-            int sbus = -1;
-            cntr = 0;
-            sbus = getSocketBus(cntr);
-            syslog(LOG_ERR, "Socket %d Bus %02x", cntr, sbus);
-            while (sbus != -1 && sbus < 0xFF)
-            {
-                sprintf(socket_bus[cntr], "%02x/", sbus);
-                cntr++;
-                sbus = getSocketBus(cntr);
-                syslog(LOG_ERR, "Socket %d Bus %02x", cntr, sbus);
-            }
-
-            if ( cntr == 0 )
-            {
-                syslog(LOG_NOTICE, "Uncore not supported on this system");
-            }
-            else
-            {
-                socket_count = cntr;
-                int fd;
-                for (int j=0; j<socket_count; j++)
-                {
-                    for (int i=1; i<MAX_NUM_PCI_DEVICES; i++)
-                    {
-                        if (pci_devices_daemon[i].path)
-                        {
-                            sprintf(pci_filepath, "%s%s%s", PCI_ROOT_PATH, socket_bus[j], pci_devices_daemon[i].path);
-                            fd = open(pci_filepath, O_RDWR);
-                            if (fd > 0)
-                            {
-                                FD_PCI[j][i] = 0;
-                                pci_devices_daemon[i].online = 1;
-                                close(fd);
-                            }
-#ifdef DEBUG_LIKWID
-                            else if (j==0)
-                            {
-                                syslog(LOG_ERR, "Device %s for socket %d not found at path %s, excluded it from device list: %s\n",pci_devices_daemon[i].name,j, pci_filepath, strerror(errno));
-                            }
-#endif
-                        }
-                    }
-                }
-            }*/
         }
     }
 LOOP:
