@@ -6,8 +6,8 @@
  *
  *      Description:  A application to retrieve and manipulate CPU features.
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -95,6 +95,9 @@ for opt,arg in likwid.getopt(arg, {"h","v","l","c:","e:","d:","a","help","versio
     elseif opt == "l" or opt == "list" then
         listFeatures = true
     elseif opt == "a" or opt == "all" then
+        if cpuinfo["isIntel"] == 0 then
+            print_stdout("INFO: Manipulation of CPU features is only available on Intel platforms")
+        end
         print_stdout("Available features:")
         for i=0,likwid.tablelength(likwid.cpuFeatures)-1 do
             local found = false
@@ -139,6 +142,11 @@ for opt,arg in likwid.getopt(arg, {"h","v","l","c:","e:","d:","a","help","versio
     end
 end
 
+if cpuinfo["isIntel"] == 0 then
+    print_stdout("INFO: Manipulation of CPU features is only available on Intel platforms")
+    os.exit(0)
+end
+
 likwid.initCpuFeatures()
 
 if listFeatures and #cpulist > 0 then
