@@ -7,8 +7,8 @@
  *      Description: A wrapper script to pin threads spawned by MPI processes and
  *                   measure hardware performance counters
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -843,7 +843,7 @@ local function assignHosts(hosts, np, ppn, tpp)
                     current = ppn*tpp
                     hosts[i] = nil
                 end
-            elseif host["slots"] then
+            elseif host["slots"]  and host["slots"] < ppn then
                 --[[if host["maxslotsno"] then
                     if host["maxslots"] < ppn then
                         print_stderr(string.format("WARN: Oversubscription for host %s needed, but max-slots set to %d.",
