@@ -7,8 +7,8 @@
  *      Description:  An application to use the timeline mode of likwid-perfctr to generate
  *                    realtime plots using feedGnuplot
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -177,13 +177,14 @@ local mfreq = 1.0
 local plotrange = 0
 local host = nil
 local force = false
+local verbose = nil
 
 if #arg == 0 then
     usage()
     os.exit(0)
 end
 
-for opt,arg in likwid.getopt(arg, {"h","v","g:","C:","c:","t:","r:","a","d","p","f","help", "version","group:","time:","dump","range:","plotdump","all", "host:", "force"}) do
+for opt,arg in likwid.getopt(arg, {"h","v","V:","g:","C:","c:","t:","r:","a","d","p","f","help", "version","group:","time:","dump","range:","plotdump","all", "host:", "force"}) do
     if opt == "h" or opt == "help" then
         usage()
         os.exit(0)
@@ -212,6 +213,11 @@ for opt,arg in likwid.getopt(arg, {"h","v","g:","C:","c:","t:","r:","a","d","p",
         host = arg
     elseif opt == "f" or opt == "force" then
         force = true
+    elseif opt == "V" then
+        local v = tonumber(arg)
+        if v >= 0 and v <= 3 then
+            verbose = v
+        end
     elseif opt == "?" then
         print_stderr("Invalid commandline option -"..arg)
         os.exit(1)
@@ -423,6 +429,9 @@ end
 if force then
     cmd = cmd .. " -f"
 end
+if verbose then
+    cmd = cmd .. string.format(" -V %d", verbose)
+end
 cmd = cmd .. string.format(" -t %s", timeline)
 
 for i, group in pairs(group_list) do
