@@ -6,8 +6,8 @@
  *
  *      Description:  An application to pin a program including threads
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -232,6 +232,9 @@ elseif num_threads > tonumber(omp_threads) and (quiet == 0 and verbose > 0) then
 end
 if omp_threads and tonumber(omp_threads) < num_threads then
     num_threads = tonumber(omp_threads)
+    for i=#cpu_list,num_threads+1,-1 do
+        cpu_list[i] = nil
+    end
 end
 
 likwid.setenv("KMP_AFFINITY","disabled")
