@@ -6,8 +6,8 @@
  *
  *      Description:  A application to set the CPU frequency of CPU cores and domains.
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -65,6 +65,10 @@ function usage()
     print_stdout("--umin freq\t Set minimal Uncore frequency")
     print_stdout("--umax freq\t Set maximal Uncore frequency")
     print_stdout("")
+    print_stdout("-reset\tSet governor 'performance', set minimal and maximal frequency to")
+    print_stdout("\tthe CPU limits and deactivate turbo")
+    print_stdout("-ureset\tSet Uncore frequencies to its min and max limits")
+    print_stdout("")
     print_stdout("For the options -f, -x and -y:")
     print_stdout("\t acpi-cpufreq driver: set the userspace governor implicitly")
     print_stdout("\t intel_pstate driver: keep current governor")
@@ -72,9 +76,14 @@ function usage()
     print_stdout("The min/max frequencies can be slightly off with the intel_pstate driver as")
     print_stdout("the value is calculated while the current frequency is read from sysfs.")
     print_stdout("")
-    print_stdout("The min/max Uncore frequency can be set freely, even to 0 or 1E20 but the hardware")
-    print_stdout("stays inside its limits. To check whether the set Uncore frequency is really set")
-    print_stdout("and fixed, use likwid-perfctr -g UNCORE_CLOCK:UBOXFIX -C 0 -S 1s")
+    print_stdout("In general the min/max Uncore frequency can be set freely, even to 0 or 1E20")
+    print_stdout("but the hardware stays inside its limits. LIKWID reduces the range of possible")
+    print_stdout("frequencies to the minimal core frequency (likwid-setFrequencies -l) and the ")
+    print_stdout("maximally achievable turbo frequency with a single core (C0 value at")
+    print_stdout("likwid-powermeter -i output).")
+    print_stdout("To check whether the set Uncore frequency is really set and fixed, use")
+    print_stdout("likwid-perfctr -g UNCORE_CLOCK:UBOXFIX -C 0 <exec_doing_work>.")
+    print_stdout("Sleeping is commonly not sufficient.")
     print_stdout("")
     print_stdout("If you switch governors with the intel_pstate driver, it might be that the driver")
     print_stdout("changes the frequency settings, please check afterwards and re-set frequencies if")
@@ -113,6 +122,7 @@ printCurFreq = false
 printAvailFreq = false
 printAvailGovs = false
 do_reset = false
+do_ureset = false
 set_turbo = false
 turbo = 0
 
@@ -122,7 +132,7 @@ if #arg == 0 then
 end
 
 
-for opt,arg in likwid.getopt(arg, {"V:", "g:", "c:", "f:", "l", "p", "h", "v", "m", "x:", "y:", "t:", "help","version","freq:", "min:", "max:", "umin:", "umax:", "reset", "turbo:"}) do
+for opt,arg in likwid.getopt(arg, {"V:", "g:", "c:", "f:", "l", "p", "h", "v", "m", "x:", "y:", "t:", "help","version","freq:", "min:", "max:", "umin:", "umax:", "reset", "turbo:", "ureset"}) do
     if opt == "h" or opt == "help" then
         usage()
         os.exit(0)
@@ -158,6 +168,8 @@ for opt,arg in likwid.getopt(arg, {"V:", "g:", "c:", "f:", "l", "p", "h", "v", "
         end
     elseif opt == "reset" then
         do_reset = true
+    elseif opt == "ureset" then
+        do_ureset = true
     elseif opt == "umin" then
         min_u_freq = arg
     elseif opt == "umax" then
@@ -235,27 +247,45 @@ end
 
 if printAvailGovs then
     local govs = likwid.getAvailGovs(0)
-    print_stdout("Available governors:")
-    print_stdout(string.format("%s", table.concat(govs, " ")))
+    if #govs > 0 then
+        print_stdout("Available governors:")
+        print_stdout(string.format("%s", table.concat(govs, " ")))
+    else
+        print_stdout("Cannot get governors from cpufreq module")
+    end
 end
 
 if printAvailFreq then
     local freqs = likwid.getAvailFreq(0)
-    print_stdout("Available frequencies:")
-    print_stdout(string.format("%s", table.concat(freqs, " ")))
+    
+    if #freqs > 0 then
+        print_stdout("Available frequencies:")
+        print_stdout(string.format("%s", table.concat(freqs, " ")))
+    else
+        print_stdout("Cannot get frequencies from cpufreq module")
+    end
 end
 
 if printCurFreq then
-    print_stdout("Current CPU frequencies:")
+    str = {"Current CPU frequencies:"}
+    local processed = 0
     for i=1,#cpulist do
         gov = likwid.getGovernor(cpulist[i])
         freq = tonumber(likwid.getCpuClockCurrent(cpulist[i]))/1E9
         min = tonumber(likwid.getCpuClockMin(cpulist[i]))/1E9
         max = tonumber(likwid.getCpuClockMax(cpulist[i]))/1E9
         t = tonumber(likwid.getTurbo(cpulist[i]));
-        print_stdout(string.format("CPU %d: governor %12s min/cur/max %s/%s/%s GHz Turbo %d",cpulist[i], gov, round(min), round(freq), round(max), t))
+        if gov and freq and min and max and t >= 0 then
+            processed = processed + 1
+            table.insert(str, string.format("CPU %d: governor %12s min/cur/max %s/%s/%s GHz Turbo %d",cpulist[i], gov, round(min), round(freq), round(max), t))
+        end
+    end
+    table.insert(str, "")
+    if processed > 0 then
+        print_stdout(table.concat(str, "\n"))
+    else
+        print_stdout("Cannot read frequency data from cpufreq module\n")
     end
-    print_stdout("")
     test = likwid.getUncoreFreqMin(socklist[i])
     if test ~= 0 then
         print_stdout("Current Uncore frequencies:")
@@ -288,7 +318,18 @@ if do_reset then
     if not governor then
         governor = availgovs[#availgovs]
     end
-    print_stdout(string.format("Reset to governor %s with min freq. %g GHz and deactivate turbo mode", governor, min_freq))
+    if min_freq and governor then
+        print_stdout(string.format("Reset to governor %s with min freq. %g GHz and deactivate turbo mode", governor, min_freq))
+    end
+end
+
+if do_ureset then
+    local availfreqs = likwid.getAvailFreq(cpulist[1])
+    local power = likwid.getPowerInfo()
+    local minf = tonumber(availfreqs[1])
+    local maxf = tonumber(power["turbo"]["steps"][1]) / 1000
+    min_u_freq = minf
+    max_u_freq = maxf
 end
 
 if numthreads > 0 and not (frequency or min_freq or max_freq or governor or min_u_freq or max_u_freq or set_turbo) then
@@ -311,6 +352,10 @@ end
 
 
 local availfreqs = likwid.getAvailFreq(cpulist[i])
+if (frequency or min_freq or max_freq) and #availfreqs == 0 then
+    print_stdout("Cannot set CPU frequency, cpufreq module not properly loaded")
+    os.exit(1)
+end
 local savailfreqs = {}
 for i,f in pairs(availfreqs) do
     savailfreqs[i] = round(f)
@@ -325,7 +370,7 @@ for x=1,2 do
         for i=1,#cpulist do
             local valid_freq = false
             for k,v in pairs(savailfreqs) do
-                if (min_freq == v) then
+                if (tonumber(min_freq) == tonumber(v)) then
                     if verbosity == 3 then
                         print_stdout(string.format("DEBUG: Min frequency %g valid", min_freq))
                     end
@@ -360,7 +405,7 @@ for x=1,2 do
         for i=1,#cpulist do
             local valid_freq = false
             for k,v in pairs(savailfreqs) do
-                if (max_freq == v) then
+                if (tonumber(max_freq) == tonumber(v)) then
                     if verbosity == 3 then
                         print_stdout(string.format("DEBUG: Max frequency %g valid", max_freq))
                     end
@@ -391,8 +436,7 @@ if min_u_freq then
         end
         local err = likwid.setUncoreFreqMin(socket, min_u_freq*1000);
         if err ~= 0 then
-            print_stderr(string.format("Setting of minimal Uncore frequency %f failed on socket %d\n", min_u_freq, socket))
-            os.exit(1)
+            print_stderr(string.format("Setting of minimal Uncore frequency %f failed on socket %d", tonumber(min_u_freq)*1000, socket))
         end
     end
 end
@@ -405,8 +449,7 @@ if max_u_freq then
         end
         local err = likwid.setUncoreFreqMax(socket, max_u_freq*1000);
         if err ~= 0 then
-            print_stderr(string.format("Setting of maximal Uncore frequency %d failed on socket %d\n", max_u_freq, socket))
-            os.exit(1)
+            print_stderr(string.format("Setting of maximal Uncore frequency %d failed on socket %d", tonumber(max_u_freq)*1000, socket))
         end
     end
 end
