@@ -9,8 +9,8 @@
  *                  Allows to turn on and off the Hardware prefetcher
  *                  available.
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
@@ -86,6 +86,10 @@ cpuFeatures_update(int cpu)
 {
     int ret;
     uint64_t flags = 0x0ULL;
+    if (cpuid_info.isIntel == 0)
+    {
+        return;
+    }
     ret = HPMread(cpu, MSR_DEV, MSR_IA32_MISC_ENABLE, &flags);
     if (ret != 0)
     {
@@ -207,7 +211,8 @@ cpuFeatures_update(int cpu)
         TEST_FLAG_INV(FEAT_HW_PREFETCHER,0);
     }
 
-    if (cpuid_info.model == XEON_PHI_KNL)
+    if ((cpuid_info.model == XEON_PHI_KNL) ||
+        (cpuid_info.model == XEON_PHI_KML))
     {
         ret = HPMread(cpu, MSR_DEV, MSR_PREFETCH_ENABLE, &flags);
         if (ret != 0)
@@ -263,7 +268,10 @@ cpuFeatures_init()
         cpuFeatureMask = malloc(cpuid_topology.numHWThreads*sizeof(uint64_t));
         memset(cpuFeatureMask, 0, cpuid_topology.numHWThreads*sizeof(uint64_t));
     }
-
+    if (cpuid_info.isIntel == 0)
+    {
+        return;
+    }
     if (!HPMinitialized())
     {
         HPMinit();
@@ -359,17 +367,23 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
         reg = MSR_PREFETCH_ENABLE;
         newOffsets = 1;
     }
-    if (cpuid_info.model == XEON_PHI_KNL)
+    if ((cpuid_info.model == XEON_PHI_KNL) ||
+        (cpuid_info.model == XEON_PHI_KML))
     {
         reg = MSR_PREFETCH_ENABLE;
         knlOffsets = 1;
         if (type == FEAT_CL_PREFETCHER ||
             type == FEAT_IP_PREFETCHER)
         {
-            fprintf(stderr, "CL_PREFETCHER and IP_PREFETCHER not available on Intel Xeon Phi (KNL)");
+            fprintf(stderr, "CL_PREFETCHER and IP_PREFETCHER not available on Intel Xeon Phi (KNL)\n");
             return 0;
         }
     }
+    if (cpuid_info.isIntel == 0)
+    {
+        fprintf(stderr, "Enabling features is only available on Intel platforms\n");
+        return -EINVAL;
+    }
 
     ret = HPMread(cpu, MSR_DEV, reg, &flags);
     if (ret != 0)
@@ -510,7 +524,8 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
         reg = MSR_PREFETCH_ENABLE;
         newOffsets = 1;
     }
-    if (cpuid_info.model == XEON_PHI_KNL)
+    if ((cpuid_info.model == XEON_PHI_KNL) ||
+        (cpuid_info.model == XEON_PHI_KML))
     {
         reg = MSR_PREFETCH_ENABLE;
         knlOffsets = 1;
@@ -521,6 +536,11 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             return 0;
         }
     }
+    if (cpuid_info.isIntel == 0)
+    {
+        fprintf(stderr, "Disabling features is only available on Intel platforms\n");
+        return -EINVAL;
+    }
     ret = HPMread(cpu, MSR_DEV, reg, &flags);
     if (ret != 0)
     {
