@@ -5,8 +5,8 @@
  *
  *      Description:  Module implementing an interface for frequency manipulation
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *                Jan Treibig (jt), jan.treibig@gmail.com
@@ -74,7 +74,7 @@ static int freq_getDriver(const int cpu_id )
     f = fopen(buff, "r");
     if (f == NULL)
     {
-        fprintf(stderr, "Unable to open path %s for reading\n", buff);
+        //fprintf(stderr, "Unable to open path %s for reading\n", buff);
         return -errno;
     }
     rptr = fgets(buff, 256, f);
@@ -119,7 +119,7 @@ static int _freq_getUncoreMinMax(const int socket_id, int *cpuId, double* min, d
     char* avail = freq_getAvailFreq(cpu);
     if (!avail)
     {
-        fprintf(stderr, "Failed to get available frequencies\n");
+        fprintf(stderr, "Failed to get available CPU frequencies\n");
         return -EINVAL;
     }
 
@@ -177,6 +177,16 @@ static int _freq_getUncoreMinMax(const int socket_id, int *cpuId, double* min, d
         *max = dmin;
         *min = dmax;
     }
+
+    power_init(cpu);
+    if (power_info.turbo.numSteps > 0)
+    {
+        if (power_info.turbo.steps[0] > *max)
+        {
+            *max = power_info.turbo.steps[0];
+        }
+    }
+
     return 0;
 }
 
@@ -222,6 +232,14 @@ uint64_t freq_getCpuClockCurrent(const int cpu_id )
     char buff[256];
     char* eptr = NULL;
     uint64_t clock = 0x0ULL;
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return 0;
+        }
+    }
 
     sprintf(buff, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_cur_freq", cpu_id);
     f = fopen(buff, "r");
@@ -245,6 +263,14 @@ uint64_t freq_setCpuClockMax(const int cpu_id, const uint64_t freq)
     char cmd[256];
     char buff[256];
     uint64_t cur = 0x0ULL;
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return 0;
+        }
+    }
 
     cur = freq_getCpuClockMax(cpu_id);
     if (cur == freq)
@@ -309,6 +335,14 @@ uint64_t freq_setCpuClockMin(const int cpu_id, const uint64_t freq)
     char cmd[256];
     char buff[256];
     uint64_t cur = 0x0ULL;
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return 0;
+        }
+    }
 
     cur = freq_getCpuClockMin(cpu_id);
     if (cur == freq)
@@ -350,6 +384,14 @@ char * freq_getGovernor(const int cpu_id )
     char cmd[256];
     char buff[256];
     char* eptr = NULL, *sptr = NULL;
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return NULL;
+        }
+    }
 
     sprintf(buff, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpu_id);
     f = fopen(buff, "r");
@@ -573,6 +615,14 @@ int freq_setTurbo(const int cpu_id, const int turbo)
 {
     FILE *fpipe = NULL;
     char cmd[256];
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return 0;
+        }
+    }
 
     sprintf(cmd, "%s %d tur %d", daemon_path, cpu_id, turbo);
     if ( !(fpipe = (FILE*)popen(cmd,"r")) )
@@ -593,6 +643,14 @@ int freq_setGovernor(const int cpu_id, const char* gov)
     FILE *fpipe = NULL;
     char cmd[256];
     char buff[256];
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return 0;
+        }
+    }
 
     sprintf(buff, "%s", daemon_path);
     if (access(buff, X_OK))
@@ -623,6 +681,14 @@ char * freq_getAvailFreq(const int cpu_id )
     double d = 0;
     int take_next = 0;
     bstring bbuff;
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return NULL;
+        }
+    }
 
     sprintf(cmd, "%s 2>&1", daemon_path);
     if ( !(fpipe = (FILE*)popen(cmd,"r")) )
@@ -685,6 +751,14 @@ char * freq_getAvailGovs(const int cpu_id )
     double d = 0;
     int take_next = 0;
     bstring bbuff;
+    if (drv == NOT_DETECTED)
+    {
+        freq_getDriver(cpu_id);
+        if (drv == NOT_DETECTED)
+        {
+            return 0;
+        }
+    }
 
     sprintf(cmd, "%s 2>&1", daemon_path);
     if ( !(fpipe = (FILE*)popen(cmd,"r")) )
@@ -731,7 +805,7 @@ int freq_setUncoreFreqMin(const int socket_id, const uint64_t freq)
     int err = 0;
     int own_hpm = 0;
     int cpuId = -1;
-    uint64_t f = freq;
+    uint64_t f = freq / 100;
     double fmin, fmax;
     if (isAMD())
     {
@@ -752,39 +826,24 @@ int freq_setUncoreFreqMin(const int socket_id, const uint64_t freq)
         ERROR_PRINT(Given frequency %llu MHz higher than system limit of %.0f MHz, freq, fmax);
         return -EINVAL;
     }
-    
+
     if (!HPMinitialized())
     {
         HPMinit();
         own_hpm = 1;
-        err = HPMaddThread(cpuId);
-        if (err != 0)
-        {
-            ERROR_PLAIN_PRINT(Cannot get access to MSRs)
-            return err;
-        }
     }
-    err = power_init(cpuId);
-    if (err < 0)
+    err = HPMaddThread(cpuId);
+    if (err != 0)
     {
-        ERROR_PRINT(Cannot initialize power module on CPU %d, cpuId);
-        return err;
-    }
-
-    if (power_info.hasRAPL)
-    {
-        f /= 100;
-    }
-    else
-    {
-        f /= 133;
+        ERROR_PLAIN_PRINT(Cannot get access to MSRs)
+        return 0;
     }
 
     uint64_t tmp = 0x0ULL;
     err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &tmp);
     if (err)
     {
-        ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
+        //ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
         return err;
     }
     tmp &= ~(0xFF00);
@@ -830,38 +889,22 @@ uint64_t freq_getUncoreFreqMin(const int socket_id)
     {
         HPMinit();
         own_hpm = 1;
-        err = HPMaddThread(cpuId);
-        if (err != 0)
-        {
-            ERROR_PLAIN_PRINT(Cannot get access to MSRs)
-            return 0;
-        }
     }
-
-    err = power_init(cpuId);
-    if (err < 0)
+    err = HPMaddThread(cpuId);
+    if (err != 0)
     {
-        ERROR_PRINT(Cannot initialize power module on CPU %d, cpuId);
+        ERROR_PLAIN_PRINT(Cannot get access to MSRs)
         return 0;
     }
 
-
     uint64_t tmp = 0x0ULL;
     err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &tmp);
     if (err)
     {
-        ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
+        //ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
         return 0;
     }
-    tmp = (tmp>>8) & 0xFFULL;
-    if (power_info.hasRAPL)
-    {
-        tmp *= 100;
-    }
-    else
-    {
-        tmp *= 133;
-    }
+    tmp = ((tmp>>8) & 0xFFULL) * 100;
 
     if (own_hpm)
         HPMfinalize();
@@ -873,8 +916,12 @@ int freq_setUncoreFreqMax(const int socket_id, const uint64_t freq)
     int err = 0;
     int own_hpm = 0;
     int cpuId = -1;
-    uint64_t f = freq;
+    uint64_t f = freq / 100;
     double fmin, fmax;
+    if (isAMD())
+    {
+        return 0;
+    }
     err = _freq_getUncoreMinMax(socket_id, &cpuId, &fmin, &fmax);
     if (err < 0)
     {
@@ -895,34 +942,19 @@ int freq_setUncoreFreqMax(const int socket_id, const uint64_t freq)
     {
         HPMinit();
         own_hpm = 1;
-        err = HPMaddThread(cpuId);
-        if (err != 0)
-        {
-            ERROR_PLAIN_PRINT(Cannot get access to MSRs)
-            return err;
-        }
     }
-    err = power_init(cpuId);
-    if (err < 0)
+    err = HPMaddThread(cpuId);
+    if (err != 0)
     {
-        ERROR_PRINT(Cannot initialize power module on CPU %d, cpuId);
-        return err;
-    }
-
-    if (power_info.hasRAPL)
-    {
-        f /= 100;
-    }
-    else
-    {
-        f /= 133;
+        ERROR_PLAIN_PRINT(Cannot get access to MSRs)
+        return 0;
     }
 
     uint64_t tmp = 0x0ULL;
     err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &tmp);
     if (err)
     {
-        ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
+        //ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
         return err;
     }
     tmp &= ~(0xFFULL);
@@ -964,17 +996,11 @@ uint64_t freq_getUncoreFreqMax(const int socket_id)
     {
         HPMinit();
         own_hpm = 1;
-        err = HPMaddThread(cpuId);
-        if (err != 0)
-        {
-            ERROR_PLAIN_PRINT(Cannot get access to MSRs)
-            return 0;
-        }
     }
-    err = power_init(cpuId);
-    if (err < 0)
+    err = HPMaddThread(cpuId);
+    if (err != 0)
     {
-        ERROR_PRINT(Cannot initialize power module on CPU %d, cpuId);
+        ERROR_PLAIN_PRINT(Cannot get access to MSRs)
         return 0;
     }
 
@@ -982,18 +1008,11 @@ uint64_t freq_getUncoreFreqMax(const int socket_id)
     err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &tmp);
     if (err)
     {
-        ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
+        //ERROR_PRINT(Cannot read register 0x%X on CPU %d, MSR_UNCORE_FREQ, cpuId);
         return 0;
     }
-    tmp = tmp & 0xFFULL;
-    if (power_info.hasRAPL)
-    {
-        tmp *= 100;
-    }
-    else
-    {
-        tmp *= 133;
-    }
+    tmp = (tmp & 0xFFULL) * 100;
+
     if (own_hpm)
         HPMfinalize();
     return tmp;
