@@ -5,8 +5,8 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -151,6 +151,8 @@ likwid_markerInit(void)
     char* eventStr = getenv("LIKWID_EVENTS");
     char* cThreadStr = getenv("LIKWID_THREADS");
     char* filepath = getenv("LIKWID_FILEPATH");
+    char* perfpid = getenv("LIKWID_PERF_EXECPID");
+    char execpid[20];
     /* Dirty hack to avoid nonnull warnings */
     int (*ownatoi)(const char*);
     ownatoi = &atoi;
@@ -218,6 +220,19 @@ likwid_markerInit(void)
         }
     }
 
+#ifdef LIKWID_USE_PERFEVENT
+    if (perfpid != NULL)
+    {
+        snprintf(execpid, 19, "%d", getpid());
+        setenv("LIKWID_PERF_PID", execpid, 1);
+        char* perfflags = getenv("LIKWID_PERF_FLAGS");
+        if (perfflags)
+        {
+            setenv("LIKWID_PERF_FLAGS", perfflags, 1);
+        }
+    }
+#endif
+
     i = perfmon_init(num_cpus, threads2Cpu);
     if (i<0)
     {
@@ -485,6 +500,8 @@ likwid_markerRegisterRegion(const char* regionTag)
         return -EFAULT;
     }
     TimerData timer;
+    int ret = 0;
+    uint64_t tmp = 0x0ULL;
     bstring tag = bfromcstralloc(100, regionTag);
     LikwidThreadResults* results;
     char groupSuffix[10];
@@ -492,11 +509,15 @@ likwid_markerRegisterRegion(const char* regionTag)
     bcatcstr(tag, groupSuffix);
     int cpu_id = hashTable_get(tag, &results);
     bdestroy(tag);
-//#ifdef LIKWID_USE_PERFEVENT
-    return HPMaddThread(cpu_id);
-//#else
-//    return 0;
-//#endif
+
+    // Add CPU to access layer
+    ret =  HPMaddThread(cpu_id);
+    // Perform one access to fully initialize connection to access daemon
+    uint32_t reg = counter_map[groupSet->groups[groups[0]].events[0].index].counterRegister;
+    HPMread(cpu_id, MSR_DEV, reg, &tmp);
+
+    return ret;
+
 }
 
 int
