@@ -5,8 +5,8 @@
  *
  *      Description:  C part of the Likwid Lua interface
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -2167,6 +2167,13 @@ lua_likwid_setVerbosity(lua_State* L)
     return 0;
 }
 
+static int
+lua_likwid_getVerbosity(lua_State* L)
+{
+    lua_pushinteger(L, (lua_Integer)(perfmon_verbosity));
+    return 1;
+}
+
 static int
 lua_likwid_access(lua_State* L)
 {
@@ -2835,6 +2842,7 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_sendSignal", lua_likwid_send_signal);
     // Verbosity functions
     lua_register(L, "likwid_setVerbosity", lua_likwid_setVerbosity);
+    lua_register(L, "likwid_getVerbosity", lua_likwid_getVerbosity); 
     // Marker API functions
     lua_register(L, "likwid_markerInit", lua_likwid_markerInit);
     lua_register(L, "likwid_markerThreadInit", lua_likwid_markerThreadInit);
