@@ -5,8 +5,8 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   4.3.1
- *      Released:  04.01.2018
+ *      Version:   4.3.2
+ *      Released:  12.04.2018
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -100,6 +100,7 @@ power_init(int cpuId)
                 case HASWELL_M1:
                 case HASWELL_M2:
                 case XEON_PHI_KNL:
+                case XEON_PHI_KML:
                     power_info.hasRAPL = 1;
                     numDomains = NUM_POWER_DOMAINS - 1;
                     break;
@@ -169,105 +170,108 @@ power_init(int cpuId)
         busSpeed = 133.33;
     }
 
-    err = HPMread(cpuId, MSR_DEV, MSR_PLATFORM_INFO, &flags);
-    if (err == 0)
+    if (cpuid_info.isIntel)
     {
-        power_info.baseFrequency = busSpeed * (double) extractBitField(flags,8,8);
-        power_info.minFrequency  = busSpeed * (double) extractBitField((flags>>(32)),8,8);
+        err = HPMread(cpuId, MSR_DEV, MSR_PLATFORM_INFO, &flags);
+        if (err == 0)
+        {
+            power_info.baseFrequency = busSpeed * (double) extractBitField(flags,8,8);
+            power_info.minFrequency  = busSpeed * (double) extractBitField((flags>>(32)),8,8);
 
-        power_info.turbo.numSteps = cpuid_topology.numCoresPerSocket;
-        if (cpuid_info.model == WESTMERE_EX)
-        {
-            power_info.turbo.numSteps = 4;
-        }
-        power_info.turbo.steps = (double*) malloc(power_info.turbo.numSteps * sizeof(double));
-        if (!power_info.turbo.steps)
-        {
-            return -ENOMEM;
-        }
-        memset(power_info.turbo.steps, 0, power_info.turbo.numSteps * sizeof(double));
-        err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT, &flags);
-        if (err)
-        {
-            fprintf(stderr,"Cannot gather values from MSR_TURBO_RATIO_LIMIT\n");
-        }
-        else
-        {
-            if (cpuid_info.model != SKYLAKEX)
+            power_info.turbo.numSteps = cpuid_topology.numCoresPerSocket;
+            if (cpuid_info.model == WESTMERE_EX)
             {
-                unsigned long flag_vals[4];
-                int flag_idx = 0;
-                int reg_idx = 1;
-                int valid_idx = 0;
-                flag_vals[0] = flags;
-                if (power_info.turbo.numSteps > 8)
-                {
-                    err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT1, &flag_vals[1]);
-                    if (err)
-                    {
-                        flag_vals[1] = 0;
-                    }
-                }
-                if (power_info.turbo.numSteps > 16)
-                {
-                    err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT2, &flag_vals[2]);
-                    if (err)
-                    {
-                        flag_vals[2] = 0;
-                    }
-                }
-                if (power_info.turbo.numSteps > 24)
-                {
-                    err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT3, &flag_vals[3]);
-                    if (err)
-                    {
-                        flag_vals[3] = 0;
-                    }
-                }
-                power_info.turbo.steps[0] = busSpeed * (double) field64(flags,0, 8);
-                for (int i=1; i < power_info.turbo.numSteps; i++)
-                {
-                    if (i % 8 == 0)
-                    {
-                        flag_idx++;
-                        reg_idx = 0;
-                    }
-                    power_info.turbo.steps[i] = busSpeed * (double) field64(flag_vals[flag_idx],(i%8)*8, 8);
-                    if (power_info.turbo.steps[i] > 0)
-                        valid_idx = i;
-                    else
-                        power_info.turbo.steps[i] = power_info.turbo.steps[valid_idx];
-                }
+                power_info.turbo.numSteps = 4;
+            }
+            power_info.turbo.steps = (double*) malloc(power_info.turbo.numSteps * sizeof(double));
+            if (!power_info.turbo.steps)
+            {
+                return -ENOMEM;
+            }
+            memset(power_info.turbo.steps, 0, power_info.turbo.numSteps * sizeof(double));
+            err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT, &flags);
+            if (err)
+            {
+                fprintf(stderr,"Cannot gather values from MSR_TURBO_RATIO_LIMIT\n");
             }
             else
             {
-                uint64_t flags_cores = 0;
-                int insert = 0;
-                err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT_CORES, &flags_cores);
-                if (err)
+                if (cpuid_info.model != SKYLAKEX)
                 {
-                    ERROR_PLAIN_PRINT(Cannot read MSR TURBO_RATIO_LIMIT_CORES);
-                    flags_cores = 0;
-                }
-                for (int i = 0; i < 8; i++)
-                {
-                    int num_cores_befores = 0;
-                    if (i > 0)
-                        num_cores_befores = field64(flags_cores,(i-1)*8, 8);
-                    int num_cores = field64(flags_cores,i*8, 8);
-                    double freq = busSpeed * (double) field64(flags, i*8, 8);
-                    for (int j = num_cores_befores; j < num_cores && insert < power_info.turbo.numSteps; j++)
+                    unsigned long flag_vals[4];
+                    int flag_idx = 0;
+                    int reg_idx = 1;
+                    int valid_idx = 0;
+                    flag_vals[0] = flags;
+                    if (power_info.turbo.numSteps > 8)
                     {
-                        power_info.turbo.steps[insert] = freq;
-                        insert++;
+                        err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT1, &flag_vals[1]);
+                        if (err)
+                        {
+                            flag_vals[1] = 0;
+                        }
+                    }
+                    if (power_info.turbo.numSteps > 16)
+                    {
+                        err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT2, &flag_vals[2]);
+                        if (err)
+                        {
+                            flag_vals[2] = 0;
+                        }
+                    }
+                    if (power_info.turbo.numSteps > 24)
+                    {
+                        err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT3, &flag_vals[3]);
+                        if (err)
+                        {
+                            flag_vals[3] = 0;
+                        }
+                    }
+                    power_info.turbo.steps[0] = busSpeed * (double) field64(flags,0, 8);
+                    for (int i=1; i < power_info.turbo.numSteps; i++)
+                    {
+                        if (i % 8 == 0)
+                        {
+                            flag_idx++;
+                            reg_idx = 0;
+                        }
+                        power_info.turbo.steps[i] = busSpeed * (double) field64(flag_vals[flag_idx],(i%8)*8, 8);
+                        if (power_info.turbo.steps[i] > 0)
+                            valid_idx = i;
+                        else
+                            power_info.turbo.steps[i] = power_info.turbo.steps[valid_idx];
+                    }
+                }
+                else
+                {
+                    uint64_t flags_cores = 0;
+                    int insert = 0;
+                    err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT_CORES, &flags_cores);
+                    if (err)
+                    {
+                        ERROR_PLAIN_PRINT(Cannot read MSR TURBO_RATIO_LIMIT_CORES);
+                        flags_cores = 0;
+                    }
+                    for (int i = 0; i < 8; i++)
+                    {
+                        int num_cores_befores = 0;
+                        if (i > 0)
+                            num_cores_befores = field64(flags_cores,(i-1)*8, 8);
+                        int num_cores = field64(flags_cores,i*8, 8);
+                        double freq = busSpeed * (double) field64(flags, i*8, 8);
+                        for (int j = num_cores_befores; j < num_cores && insert < power_info.turbo.numSteps; j++)
+                        {
+                            power_info.turbo.steps[insert] = freq;
+                            insert++;
+                        }
                     }
                 }
             }
         }
-    }
-    else
-    {
-        fprintf(stderr,"Cannot gather values from MSR_PLATFORM_INFO\n");
+        else
+        {
+            fprintf(stderr,"Cannot gather values from MSR_PLATFORM_INFO\n");
+        }
     }
 
     /* determine RAPL parameters */
@@ -303,7 +307,8 @@ power_init(int cpuId)
                 (cpuid_info.model == BROADWELL_D) ||
                 (cpuid_info.model == BROADWELL_E) ||
                 (cpuid_info.model == SKYLAKEX) ||
-                (cpuid_info.model == XEON_PHI_KNL))
+                (cpuid_info.model == XEON_PHI_KNL) ||
+                (cpuid_info.model == XEON_PHI_KML))
             {
                 power_info.domains[DRAM].energyUnit = 15.3E-6;
             }
